@@ -28,7 +28,7 @@ class OptionsBarClassic(AbstractOptionsBar):
 	def __init__(self, window):
 		super().__init__()
 		self.window = window
-		builder = self.build_ui('optionsbars/classic/optionsbar-classic.ui')
+		builder = self._build_ui('classic/optionsbar-classic.ui')
 
 		self.color_box = builder.get_object('color_box')
 		self.color_menu_btn_r = builder.get_object('color_menu_btn_r')
@@ -39,20 +39,23 @@ class OptionsBarClassic(AbstractOptionsBar):
 		window.add_action_enum('cairo_op_mirror', 'over', self._cairop_mirror)
 		self._cairo_operator_lock = False
 
-		self.options_btn = builder.get_object('options_btn')
 		self.options_label = builder.get_object('options_label')
 		self.options_long_box = builder.get_object('options_long_box')
 		self.options_short_box = builder.get_object('options_short_box')
 
 		self.thickness_scalebtn = builder.get_object('thickness_scalebtn')
 		self.thickness_spinbtn = builder.get_object('thickness_spinbtn')
-		self.thickness_spinbtn.set_value(window._settings.get_int('last-size'))
+		last_size = self._get_tool_options().get_int('last-size')
+		self.thickness_spinbtn.set_value(last_size)
 		utilities_add_unit_to_spinbtn(self.thickness_spinbtn, 3, 'px')
 
 		self.minimap_btn = builder.get_object('minimap_btn')
 		self.minimap_label = builder.get_object('minimap_label')
 		self.minimap_arrow = builder.get_object('minimap_arrow')
 
+	def _get_tool_options(self):
+		return self.window.options_manager._tools_gsettings
+
 	def update_for_new_tool(self, tool):
 		self.color_box.set_sensitive(tool.use_color)
 		if tool.use_operator:
@@ -70,12 +73,6 @@ class OptionsBarClassic(AbstractOptionsBar):
 	def set_minimap_label(self, label):
 		self.minimap_label.set_label(label)
 
-	def toggle_options_menu(self):
-		self.options_btn.set_active(not self.options_btn.get_active())
-
-	def hide_options_menu(self):
-		self.options_btn.set_active(False)
-
 	def build_options_menu(self, widget, model, label):
 		if widget is not None:
 			self.options_btn.set_popover(widget)
@@ -92,7 +89,7 @@ class OptionsBarClassic(AbstractOptionsBar):
 		           self.options_long_box.get_preferred_width()[0] + \
 		                self.minimap_btn.get_preferred_width()[0] + 50
 		# assuming 50px is enough to compensate the length of the label
-		self.set_limit_size(temp_limit_size)
+		self._set_limit_size(temp_limit_size)
 
 	def set_compact(self, state):
 		super().set_compact(state)
@@ -113,8 +110,8 @@ class OptionsBarClassic(AbstractOptionsBar):
 	def _build_color_buttons(self, builder):
 		"""Initialize the 2 color-buttons and popovers with the 2 previously
 		memorized RGBA values."""
-		right_rgba = self.window._settings.get_strv('last-right-rgba')
-		left_rgba = self.window._settings.get_strv('last-left-rgba')
+		right_rgba = self._get_tool_options().get_strv('last-right-rgba')
+		left_rgba = self._get_tool_options().get_strv('last-left-rgba')
 		self._color_r = OptionsBarClassicColorPopover(self.color_menu_btn_r, \
 		             builder.get_object('r_btn_image'), right_rgba, False, self)
 		self._color_l = OptionsBarClassicColorPopover(self.color_menu_btn_l, \
