@@ -16,8 +16,9 @@
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 # Import libs
-import os
+import os, traceback
 from gi.repository import Gtk, Gdk, Gio, GdkPixbuf, GLib
+from .gi_composites import GtkTemplate
 
 # Import tools
 from .tool_arc import ToolArc
@@ -76,27 +77,28 @@ DEFAULT_TOOL_ID = 'pencil'
 
 ################################################################################
 
-@Gtk.Template(resource_path=UI_PATH+'window.ui')
+@GtkTemplate(ui=UI_PATH+'window.ui')
 class DrWindow(Gtk.ApplicationWindow):
 	__gtype_name__ = 'DrWindow'
 
 	gsettings = Gio.Settings.new('com.github.maoschanz.drawing')
 
 	# Window empty widgets
-	tools_flowbox = Gtk.Template.Child()
-	toolbar_box = Gtk.Template.Child()
-	info_bar = Gtk.Template.Child()
-	info_label = Gtk.Template.Child()
-	info_action = Gtk.Template.Child()
-	notebook = Gtk.Template.Child()
-	bottom_panes_box = Gtk.Template.Child()
-	unfullscreen_btn = Gtk.Template.Child()
-	bottom_meta_box = Gtk.Template.Child()
-	tools_scrollable_box = Gtk.Template.Child()
-	tools_nonscrollable_box = Gtk.Template.Child()
+	tools_flowbox = GtkTemplate.Child()
+	toolbar_box = GtkTemplate.Child()
+	info_bar = GtkTemplate.Child()
+	info_label = GtkTemplate.Child()
+	info_action = GtkTemplate.Child()
+	notebook = GtkTemplate.Child()
+	bottom_panes_box = GtkTemplate.Child()
+	unfullscreen_btn = GtkTemplate.Child()
+	bottom_meta_box = GtkTemplate.Child()
+	tools_scrollable_box = GtkTemplate.Child()
+	tools_nonscrollable_box = GtkTemplate.Child()
 
 	def __init__(self, **kwargs):
 		super().__init__(**kwargs)
+		self.init_template()
 		self.app = kwargs['application']
 
 		self.pointer_to_current_page = None # this ridiculous hack allows to
@@ -104,24 +106,40 @@ class DrWindow(Gtk.ApplicationWindow):
 		                                      # notebook widget being pure shit
 		self.active_tool_id = None
 		self._is_tools_initialisation_finished = False
+		self.devel_mode = False
+		self.should_track_framerate = False
 
 		if self.gsettings.get_boolean('maximized'):
 			self.maximize()
+
+		self._update_theme_variant()
 		# self.resize(360, 648)
 		# self.resize(720, 288)
-		self.set_ui_bars()
+		self._set_ui_bars()
 
-	def init_window_content(self, gfile, get_cb):
+	def init_window_content_async(self, content_params):
 		"""Initialize the window's content, such as the minimap, the color
 		popovers, the tools, their options, and a new image. Depending on the
 		parameters, the new image can be imported from the clipboard, loaded
-		from a GioFile, or (else) it can be a blank image."""
-		self.tools = None
+		from a GioFile, or (else) it can be a blank image.
+
+		This method is called asynchronously, which isn't *correct* (not very
+		thread-safe or anything) but it allows the window to be shown quicker.
+		If it fails, a window is here anyway because this is independant from
+		the object constructor."""
+
+		self.prompt_action(_("Error starting the application, please report this bug."))
+
+		gfile = content_params['gfile']
+		get_cb = content_params['get_cb']
+
+		self.tools = {}
 		self.minimap = DrMinimap(self, None)
 		self.options_manager = DrOptionsManager(self)
 		self.saving_manager = DrSavingManager(self)
 		self.printing_manager = DrPrintingManager(self)
 
+		self.devel_mode = self.gsettings.get_boolean('devel-only')
 		self.add_all_win_actions()
 		self._init_tools()
 		self.connect_signals()
@@ -132,7 +150,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		# can continue normally.
 		try:
 			if get_cb:
-				self.build_image_from_clipboard()
+				self.delayed_build_from_clipboard()
 			elif gfile is not None:
 				self.build_new_tab(gfile=gfile)
 			else:
@@ -144,6 +162,9 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.set_picture_title()
 		self._try_show_release_notes()
 
+		# has to return False to be removed from the mainloop immediatly
+		return False
+
 	def _try_show_release_notes(self):
 		last_version = self.gsettings.get_string('last-version')
 		current_version = self.app.get_current_version()
@@ -229,14 +250,13 @@ class DrWindow(Gtk.ApplicationWindow):
 	def _load_tool(self, tool_id, tool_class, disabled_tools, dev):
 		"""Given its id and its python class, this method tries to load a tool,
 		and show an error message if the tool initialization failed."""
-		if dev: # Simplest way to get an error stack
-			self.tools[tool_id] = tool_class(self)
-		elif tool_id not in disabled_tools:
+		if tool_id not in disabled_tools:
 			try:
 				self.tools[tool_id] = tool_class(self)
-			except:
+			except Exception as err:
 				# Context: an error message
-				self.prompt_message(True, _("Failed to load tool: %s") % tool_id)
+				self.prompt_action(_("Failed to load tool: %s") % tool_id)
+				traceback.print_exc()
 
 	def _build_tool_rows(self):
 		"""Adds each tool's button to the side pane."""
@@ -294,6 +314,16 @@ class DrWindow(Gtk.ApplicationWindow):
 			self.set_picture_title()
 		dialog.destroy()
 
+	def delayed_build_from_clipboard(self, *args):
+		"""Calls `async_build_from_clipboard` asynchronously."""
+		self.build_new_tab() # temporary image to avoid errors when the window
+		# finishes its initialisation.
+		GLib.timeout_add(500, self.async_build_from_clipboard, {})
+
+	def async_build_from_clipboard(self, content_params):
+		self.get_active_image().try_close_tab()
+		self.build_image_from_clipboard()
+
 	def build_image_from_clipboard(self, *args):
 		"""Open a new tab with the image in the clipboard. If the clipboard is
 		empty, the new image will be blank."""
@@ -311,7 +341,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.build_new_tab(pixbuf=pixbuf)
 
 	def build_new_tab(self, gfile=None, pixbuf=None, \
-		           width=200, height=200, background_rgba=[1.0, 0.0, 0.0, 1.0]):
+		           width=200, height=200, background_rgba=[0.5, 0.5, 0.5, 0.5]):
 		"""Open a new tab with an optional file to open in it."""
 		new_image = DrImage(self)
 		self.notebook.append_page(new_image, new_image.build_tab_widget())
@@ -415,9 +445,10 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.gsettings.connect('changed::show-labels', self.on_show_labels_setting_changed)
 		self.gsettings.connect('changed::deco-type', self.on_layout_changed)
 		self.gsettings.connect('changed::big-icons', self.on_icon_size_changed)
-		# self.gsettings.connect('changed::preview-size', self.show_info_settings)
-		# self.gsettings.connect('changed::devel-only', self.show_info_settings)
+		self.gsettings.connect('changed::preview-size', self.show_info_settings)
+		# devel-only' isn't connected because it's auto-updated to False
 		self.gsettings.connect('changed::disabled-tools', self.show_info_settings)
+		self.gsettings.connect('changed::dark-theme-variant', self._update_theme_variant)
 		# Other settings are connected in DrImage
 
 		# What happens when the active image change
@@ -536,10 +567,16 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.add_action_simple('paste', self.action_paste, ['<Ctrl>v'])
 		self.add_action_simple('select_all', self.action_select_all, ['<Ctrl>a'])
 		self.add_action_simple('unselect', self.action_unselect, ['<Ctrl><Shift>a'])
+		#self.add_action_simple('selection_invert', self.action_selection_invert, None)
 		self.add_action_simple('selection_cut', self.action_cut, ['<Ctrl>x'])
 		self.add_action_simple('selection_copy', self.action_copy, ['<Ctrl>c'])
 		self.add_action_simple('selection_delete', self.action_delete, ['Delete'])
+
 		self.add_action_simple('selection_export', self.action_selection_export, None)
+		self.add_action_simple('selection-replace-canvas', \
+		                             self.action_selection_replace_canvas, None)
+		self.add_action_simple('selection-expand-canvas', \
+		                              self.action_selection_expand_canvas, None)
 
 		self.add_action_simple('back_to_previous', self.back_to_previous, ['<Ctrl>b'])
 		self.add_action_simple('force_selection', self.force_selection, None)
@@ -557,10 +594,11 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.add_action_simple('size_more', self.action_size_more, ['<Ctrl><Shift>Up'])
 		self.add_action_simple('size_less', self.action_size_less, ['<Ctrl><Shift>Down'])
 
-		if self.gsettings.get_boolean('devel-only'):
+		if self.devel_mode:
 			self.add_action_simple('restore_pixbuf', self.action_restore, None)
 			self.add_action_simple('rebuild_from_histo', self.action_rebuild, None)
 			self.add_action_simple('get_values', self.action_getvalues, ['<Ctrl>g'])
+			self.add_action_boolean('track_framerate', False, self.action_fsp)
 
 		action = Gio.PropertyAction.new('active_tab', self.notebook, 'page')
 		self.add_action(action)
@@ -582,7 +620,7 @@ class DrWindow(Gtk.ApplicationWindow):
 	def on_layout_changed(self, *args):
 		try:
 			is_narrow = self._decorations.remove_from_ui()
-			self.set_ui_bars()
+			self._set_ui_bars()
 			self._decorations.set_compact(is_narrow)
 			self.set_picture_title()
 		except:
@@ -628,7 +666,7 @@ class DrWindow(Gtk.ApplicationWindow):
 			return 'hg' # Use the GNOME layout if the desktop is unknown,
 		# because i don't know how the env variable is on mobile.
 
-	def set_ui_bars(self):
+	def _set_ui_bars(self):
 		"""Set the UI "bars" (headerbar, menubar, titlebar, toolbar, whatever)
 		according to the user's preference, which by default is an empty string.
 		In this case, an useful string is set by `get_auto_decorations()`."""
@@ -657,7 +695,7 @@ class DrWindow(Gtk.ApplicationWindow):
 			self._decorations = DrDecoManagerToolbar(symbolic, menubar, self)
 		else:
 			self.gsettings.set_string('deco-type', '')
-			self.set_ui_bars() # yes, recursion.
+			self._set_ui_bars() # yes, recursion.
 
 		if self.app.is_beta():
 			self.get_style_context().add_class('devel')
@@ -667,9 +705,7 @@ class DrWindow(Gtk.ApplicationWindow):
 
 	def action_options_menu(self, *args):
 		"""This displays/hides the tool's options menu, and is implemented as an
-		action to ease the accelerator (shift+f10). This action could be
-		disable when the current pane doesn't contain the corresponding button,
-		but will not be."""
+		action to ease the accelerator (shift+f10)."""
 		self.options_manager.toggle_menu()
 
 	def _adapt_to_window_size(self, *args):
@@ -699,10 +735,10 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.info_action.set_visible(False)
 		if show:
 			self.info_label.set_label(label)
-		if show or self.gsettings.get_boolean('devel-only') and label != "":
-			print('Drawing: ' + label)
+		if show and self.devel_mode and label != "":
+			print("Drawing: " + label)
 
-	def prompt_action(self, message, action_name, action_label):
+	def prompt_action(self, message, action_name='app.report_bug', action_label=_("Report a bug")):
 		"""Update the content of the info bar, including its actionable button
 		which is set as visible."""
 		self.prompt_message(True, message)
@@ -715,6 +751,11 @@ class DrWindow(Gtk.ApplicationWindow):
 		should_show = (self.notebook.get_n_pages() > 1) and not controls_hidden
 		self.notebook.set_show_tabs(should_show)
 
+	def _update_theme_variant(self, *args):
+		key = 'gtk-application-prefer-dark-theme';
+		use_dark_theme = self.gsettings.get_boolean('dark-theme-variant')
+		Gtk.Settings.get_default().set_property(key, use_dark_theme)
+
 	############################################################################
 	# FULLSCREEN ###############################################################
 
@@ -733,6 +774,7 @@ class DrWindow(Gtk.ApplicationWindow):
 			self.prompt_message(True, _("Middle-click, tap with 3 fingers, " + \
 			                           "or press F8 to show/hide controls.") + \
 			                           " " + _("Press F11 to exit fullscreen."))
+			# TODO à confirmer que 3 doigts c'est réel ^
 		else:
 			self.unfullscreen()
 		self._set_controls_hidden(shall_fullscreen)
@@ -868,7 +910,8 @@ class DrWindow(Gtk.ApplicationWindow):
 		if self.former_tool_id == self.active_tool_id:
 			self.force_selection()
 			# avoid cases where applying a transform tool keeps the tool active
-		self.tools[self.former_tool_id].row.set_active(True)
+		else:
+			self.tools[self.former_tool_id].row.set_active(True)
 
 	def _build_options_menu(self):
 		"""Build the active tool's option menus.
@@ -919,6 +962,13 @@ class DrWindow(Gtk.ApplicationWindow):
 		done here but it's done in DrImage to have a satisfying UML diagram."""
 		self.get_active_image().show_properties()
 
+	def action_fsp(self, *args):
+		"""Development only: tracks and displays the framerate, thus it helps
+		debugging how Gdk/cairo draws on the widget."""
+		self.should_track_framerate = not self.should_track_framerate
+		for img in self.notebook.get_children():
+			img.reset_fps_counter()
+
 	def get_active_image(self):
 		if self.pointer_to_current_page is None:
 			return self.notebook.get_nth_page(self.notebook.get_current_page())
@@ -944,9 +994,7 @@ class DrWindow(Gtk.ApplicationWindow):
 			self.try_load_file(gfile)
 		else:
 			dialog = DrMessageDialog(self)
-			# Context: answer to "where do you want to open the image?"
 			new_tab_id = dialog.set_action(_("New Tab"), None, True)
-			# Context: answer to "where do you want to open the image?"
 			new_window_id = dialog.set_action(_("New Window"), None, False)
 			discard_id = dialog.set_action(_("Discard changes"), \
 			                                        'destructive-action', False)
@@ -1122,6 +1170,17 @@ class DrWindow(Gtk.ApplicationWindow):
 	def action_selection_export(self, *args):
 		return self.saving_manager.save_current_image(True, True, True, True)
 
+	def action_selection_replace_canvas(self, *args):
+		self.get_selection_tool().replace_canvas()
+
+	def action_selection_expand_canvas(self, *args):
+		crop_tool = self.tools['crop']
+		operation = crop_tool.build_selection_fit_operation()
+		crop_tool.apply_operation(operation) # calling this here isn't elegant
+
+	def action_selection_invert(self, *args):
+		self.get_selection_tool().invert_selection()
+
 	def get_selection_tool(self):
 		if 'rect_select' in self.tools:
 			return self.tools['rect_select']
